@@ -2,7 +2,9 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019, 2020, 2021, 2022 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -178,7 +180,11 @@ USA.
   ;; Attempt to reuse source for target if it is in ST(0).
   ;; Otherwise we will target ST(0) by sorting the machine registers.
   (cond ((and (pseudo-register? target) (pseudo-register? source)
-	      (eqv? fr0 (pseudo-register-alias *register-map* 'FLOAT source)))
+	      (eqv? fr0
+		    (pseudo-register-alias *register-map*
+					   'FLOAT
+					   source
+					   *target-machine-registers*)))
 	 (reuse-pseudo-register-alias
 	  source 'FLOAT
 	  (lambda (alias)
@@ -221,7 +227,10 @@ USA.
   (cond ((pseudo-register? target)
 	 (let ((alias
 		(and (dead-register? source)
-		     (pseudo-register-alias *register-map* 'FLOAT source))))
+		     (pseudo-register-alias *register-map*
+					    'FLOAT
+					    source
+					    *target-machine-registers*))))
 	   (if alias
 	       (default)))
 	
@@ -265,11 +274,7 @@ USA.
   ;; Disabled: FSIN and FCOS limited to pi * 2^62.
   ;;(define-flonum-operation FLONUM-SIN FSIN)
   ;;(define-flonum-operation FLONUM-COS FCOS)
-  (define-flonum-operation FLONUM-SQRT FSQRT)
-  (define-flonum-operation FLONUM-ROUND FRNDINT))
-
-;; These (and FLONUM-ROUND above) presume that the default rounding mode
-;; is round-to-nearest/even
+  (define-flonum-operation FLONUM-SQRT FSQRT))
 
 (define (define-rounding prim-name mode)
   (define-arithmetic-method prim-name flonum-methods/1-arg
@@ -281,6 +286,7 @@ USA.
 		    (LAP)
 		    (LAP (FLD (ST ,source))))
 	      (MOV B ,temp (@RO B ,regnum:free-pointer 1))
+	      (AND B (@RO B ,regnum:free-pointer 1) (&U #xf3))
 	      (OR B (@RO B ,regnum:free-pointer 1) (&U ,mode))
 	      (FNLDCW (@R ,regnum:free-pointer))
 	      (FRNDINT)
@@ -290,6 +296,7 @@ USA.
 		    (LAP (FSTP (ST ,(1+ target)))))
 	      (FNLDCW (@R ,regnum:free-pointer))))))))
 
+(define-rounding 'FLONUM-ROUND #x00)
 (define-rounding 'FLONUM-CEILING #x08)
 (define-rounding 'FLONUM-FLOOR #x04)
 (define-rounding 'FLONUM-TRUNCATE #x0c)
@@ -319,6 +326,27 @@ USA.
 	  (FXCH (ST 0) (ST 1))
 	  (FYL2X)))))
 
+(define-arithmetic-method 'FLONUM-LOG1P flonum-methods/1-arg
+  ;; Computes LOG(X+1).
+  ;; X must be in the range: (- (SQRT 1/2) 1) <= X <= (- 1 (SQRT 1/2))
+  (flonum-unary-operation/stack-top
+   (lambda ()
+     (LAP (FLDLN2)
+	  (FXCH (ST 0) (ST 1))
+	  (FYL2XP1)))))
+
+(define-arithmetic-method 'FLONUM-EXPM1 flonum-methods/1-arg
+  ;; Computes EXP(X)-1.
+  ;; X must be in the range: (- (LOG 2)) <= X <= (LOG 2)
+  (flonum-unary-operation/stack-top
+   (lambda ()
+     (LAP (FLDL2E)
+	  (FMULP (ST 1) (ST 0))
+	  (F2XM1)))))
+
+;; XXX This is disabled because it is inadequate in 53-bit precision.
+
+#;
 (define-arithmetic-method 'FLONUM-EXP flonum-methods/1-arg
   (flonum-unary-operation/stack-top
    (lambda ()
@@ -630,7 +658,17 @@ USA.
 
 (define-rule predicate
   (FLONUM-PRED-1-ARG (? predicate) (REGISTER (? source)))
-  (flonum-compare-zero predicate source))
+  (flonum-compare-zero (flonum-pred-1->2-args predicate) source))
+
+;;; For a predicate giving (if (predicate x) a b), return predicate* so
+;;; that (if (predicate* x 0) a b) is equivalent.
+
+(define (flonum-pred-1->2-args predicate)
+  (case predicate
+    ((FLONUM-ZERO?) 'FLONUM-EQUAL?)
+    ((FLONUM-NEGATIVE?) 'FLONUM-LESS?)
+    ((FLONUM-POSITIVE?) 'FLONUM-GREATER?)
+    (else (error "Invalid flonum-pred-1-arg:" predicate))))
 
 (define-rule predicate
   (FLONUM-PRED-2-ARGS (? predicate)
@@ -640,14 +678,20 @@ USA.
 	 (st2 (flonum-source! source2)))
     (cond ((zero? st1)
 	   (flonum-branch! predicate
-			   (LAP (FCOM (ST 0) (ST ,st2)))))
+			   (lambda (fcom fcomp)
+			     fcomp
+			     (LAP (,fcom (ST 0) (ST ,st2))))))
 	  ((zero? st2)
 	   (flonum-branch! (commute-flonum-predicate predicate)
-			   (LAP (FCOM (ST 0) (ST ,st1)))))
+			   (lambda (fcom fcomp)
+			     fcomp
+			     (LAP (,fcom (ST 0) (ST ,st1))))))
 	  (else
 	   (flonum-branch! predicate
-			   (LAP (FLD (ST ,st1))
-				(FCOMP (ST 0) (ST ,(1+ st2)))))))))
+			   (lambda (fcom fcomp)
+			     fcom
+			     (LAP (FLD (ST ,st1))
+				  (,fcomp (ST 0) (ST ,(1+ st2))))))))))
 
 (define-rule predicate
   (FLONUM-PRED-2-ARGS (? predicate)
@@ -675,30 +719,52 @@ USA.
 
 (define (flonum-compare-zero predicate source)
   (let ((sti (flonum-source! source)))
-    (if (zero? sti)
-	(flonum-branch! predicate
-			(LAP (FTST)))
-	(flonum-branch! (commute-flonum-predicate predicate)
-			(LAP (FLDZ)
-			     (FCOMP (ST 0) (ST ,(1+ sti))))))))
+    (flonum-branch! (commute-flonum-predicate predicate)
+		    (lambda (fcom fcomp)
+		      fcom
+		      (LAP (FLDZ)
+			   (,fcomp (ST 0) (ST ,(1+ sti))))))))
 
 (define (flonum-compare-one predicate source)
   (let ((sti (flonum-source! source)))
     (flonum-branch! (commute-flonum-predicate predicate)
-		    (LAP (FLD1)
-			 (FCOMP (ST 0) (ST ,(1+ sti)))))))
+		    (lambda (fcom fcomp)
+		      fcom
+		      (LAP (FLD1)
+			   (,fcomp (ST 0) (ST ,(1+ sti))))))))
+
+;;; For predicate giving (if (predicate x y) a b), return predicate* so
+;;; that (if (predicate* y x) a b) is equivalent.
 
 (define (commute-flonum-predicate pred)
   (case pred
-    ((FLONUM-EQUAL? FLONUM-ZERO?) 'FLONUM-EQUAL?)
-    ((FLONUM-LESS? FLONUM-NEGATIVE?) 'FLONUM-GREATER?)
-    ((FLONUM-GREATER? FLONUM-POSITIVE?) 'FLONUM-LESS?)
+    ((FLONUM-EQUAL?) 'FLONUM-EQUAL?)
+    ((FLONUM-LESS?) 'FLONUM-GREATER?)
+    ((FLONUM-GREATER?) 'FLONUM-LESS?)
+    ((FLONUM-IS-EQUAL?) 'FLONUM-IS-EQUAL?)
+    ((FLONUM-IS-LESS?) 'FLONUM-IS-GREATER?)
+    ((FLONUM-IS-LESS-OR-EQUAL?) 'FLONUM-IS-GREATER-OR-EQUAL?)
+    ((FLONUM-IS-GREATER?) 'FLONUM-IS-LESS?)
+    ((FLONUM-IS-GREATER-OR-EQUAL?) 'FLONUM-IS-LESS-OR-EQUAL?)
+    ((FLONUM-IS-UNORDERED?) 'FLONUM-IS-UNORDERED?)
+    ((FLONUM-IS-LESS-OR-GREATER?) 'FLONUM-IS-LESS-OR-GREATER?)
     (else
      (error "commute-flonum-predicate: Unknown predicate" pred))))
 
 (define (flonum-branch! predicate prefix)
+  (define (comparison fcom fcomp)
+    (flush-register! eax)
+    (LAP ,@(prefix fcom fcomp)
+	 ;; FCOMI &c. are not supported by all IA-32 CPUs -- only >=P6.
+	 ;; So explicitly convert condition codes via the status word.
+	 (FSTSW (R ,eax))
+	 (SAHF)))
+  (define (quiet-comparison)
+    (comparison 'FUCOM 'FUCOMP))
+  (define (signalling-comparison)
+    (comparison 'FCOM 'FCOMP))
   (case predicate
-    ((FLONUM-EQUAL? FLONUM-ZERO?)
+    ((FLONUM-EQUAL? FLONUM-IS-EQUAL?)
      (set-current-branches! (lambda (label)
 			      (let ((unordered (generate-label 'UNORDERED)))
 				(LAP (JP (@PCR ,unordered))
@@ -706,8 +772,12 @@ USA.
 				     (LABEL ,unordered))))
 			    (lambda (label)
 			      (LAP (JNE (@PCR ,label))
-				   (JP (@PCR ,label))))))
-    ((FLONUM-LESS? FLONUM-NEGATIVE?)
+				   (JP (@PCR ,label)))))
+     (case predicate
+       ((FLONUM-EQUAL?) (signalling-comparison))
+       ((FLONUM-IS-EQUAL?) (quiet-comparison))
+       (else (assert #f))))
+    ((FLONUM-LESS? FLONUM-IS-LESS?)
      (set-current-branches! (lambda (label)
 			      (let ((unordered (generate-label 'UNORDERED)))
 				(LAP (JP (@PCR ,unordered))
@@ -715,18 +785,50 @@ USA.
 				     (LABEL ,unordered))))
 			    (lambda (label)
 			      (LAP (JAE (@PCR ,label))
-				   (JP (@PCR ,label))))))
-    ((FLONUM-GREATER? FLONUM-POSITIVE?)
+				   (JP (@PCR ,label)))))
+     (case predicate
+       ((FLONUM-LESS?) (signalling-comparison))
+       ((FLONUM-IS-LESS?) (quiet-comparison))
+       (else (assert #f))))
+    ((FLONUM-GREATER? FLONUM-IS-GREATER?)
      (set-current-branches! (lambda (label)
 			      (LAP (JA (@PCR ,label))))
 			    (lambda (label)
-			      (LAP (JBE (@PCR ,label))))))
+			      (LAP (JBE (@PCR ,label)))))
+     (case predicate
+       ((FLONUM-GREATER?) (signalling-comparison))
+       ((FLONUM-IS-GREATER?) (quiet-comparison))
+       (else (assert #f))))
+    ((FLONUM-IS-LESS-OR-EQUAL?)
+     (set-current-branches! (lambda (label)
+                              (let ((unordered (generate-label 'UNORDERED)))
+                                (LAP (JP (@PCR ,unordered))
+                                     (JBE (@PCR ,label))
+                                     (LABEL ,unordered))))
+                            (lambda (label)
+                              (LAP (JA (@PCR ,label))
+                                   (JP (@PCR ,label)))))
+     (quiet-comparison))
+    ((FLONUM-IS-GREATER-OR-EQUAL?)
+     (set-current-branches! (lambda (label)
+                              (LAP (JAE (@PCR ,label))))
+                            (lambda (label)
+                              (LAP (JB (@PCR ,label)))))
+     (quiet-comparison))
+    ((FLONUM-IS-LESS-OR-GREATER?)
+     (set-current-branches! (lambda (label)
+                              (LAP (JNE (@PCR ,label))))
+                            (lambda (label)
+                              (LAP (JE (@PCR ,label)))))
+     (quiet-comparison))
+    ((FLONUM-IS-UNORDERED?)
+     (set-current-branches! (lambda (label)
+                              (LAP (JP (@PCR ,label))))
+                            (lambda (label)
+                              (LAP (JNP (@PCR ,label)))))
+     (quiet-comparison))
     (else
-     (error "flonum-branch!: Unknown predicate" predicate)))
-  (flush-register! eax)
-  (LAP ,@prefix
-       (FSTSW (R ,eax))
-       (SAHF)))
+     (error "flonum-branch!: Unknown predicate" predicate))))
 
 ;; This is endianness dependent!
 
@@ -792,12 +894,14 @@ USA.
   (flonum->label fp-value 'SINGLE-FLOATS 4 0
 		 (LAP ,@(lap:comment `(SINGLE-FLOAT ,fp-value))
 		      (LONG U ,(flo:32-bit-representation-exact? fp-value)))))
-				     
+
 (define-rule statement
   (ASSIGN (REGISTER (? target)) (OBJECT->FLOAT (CONSTANT (? fp-value))))
   (cond ((not (flo:flonum? fp-value))
 	 (error "OBJECT->FLOAT: Not a floating-point value" fp-value))
-	((flo:= fp-value 0.0)
+	((and (flo:= fp-value 0.0)
+              ;; XXX Kludgey but expedient test for zero sign.
+              (not (flo:negative? (flo:atan2 fp-value -1.))))
 	 (let ((target (flonum-target! target)))
 	   (LAP (FLDZ)
 		(FSTP (ST ,(1+ target))))))
