@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019, 2020, 2021, 2022 Massachusetts Institute of
     Technology
 
 This file is part of MIT/GNU Scheme.
@@ -125,7 +126,7 @@ USA.
 
 (define (rtl:make-push-return continuation)
   (rtl:make-push
-   (rtl:make-cons-pointer (rtl:make-machine-constant type-code:compiled-entry)
+   (rtl:make-cons-pointer (rtl:make-machine-constant type-code:compiled-return)
 			  (rtl:make-entry:continuation continuation))))
 
 (define (rtl:make-push-link)
@@ -226,17 +227,17 @@ USA.
   (expression-simplify expression scfg*pcfg->pcfg! receiver))
 
 (define-export (expression-simplify-for-pseudo-assignment expression receiver)
-  (let ((entry (assq (car expression) expression-methods)))
+  (let ((entry (hash-table-ref/default expression-methods (car expression) #f)))
     (if entry
-	(apply (cdr entry) receiver scfg*scfg->scfg! (cdr expression))
+	(apply entry receiver scfg*scfg->scfg! (cdr expression))
 	(receiver expression))))
 
 (define (expression-simplify expression scfg-append! receiver)
   (if (rtl:register? expression)
       (receiver expression)
-      (let ((entry (assq (car expression) expression-methods)))
+      (let ((entry (hash-table-ref/default expression-methods (car expression) #f)))
 	(if entry
-	    (apply (cdr entry)
+	    (apply entry
 		   (lambda (expression)
 		     (if (rtl:register? expression)
 			 (receiver expression)
@@ -360,7 +361,7 @@ USA.
 			granularity*)
 		       scfg-append!
 		       (lambda (base-reg)
-			(if-memory base-reg offset granularity)))))) 
+			(if-memory base-reg offset granularity))))))
 		  (else
 		   (error "illegal offset base" locative)))))
 	     ((INDEX)
@@ -409,15 +410,11 @@ USA.
      (receiver pseudo))))
 
 (define (define-expression-method name method)
-  (let ((entry (assq name expression-methods)))
-    (if entry
-	(set-cdr! entry method)
-	(set! expression-methods
-	      (cons (cons name method) expression-methods))))
+  (hash-table-set! expression-methods name method)
   name)
 
 (define expression-methods
-  '())
+  (make-strong-eq-hash-table))
 
 (define-expression-method 'FETCH
   (lambda (receiver scfg-append! locative)
@@ -437,7 +434,7 @@ USA.
 (define-expression-method 'ADDRESS
   (address-method
    (lambda (receiver scfg-append!)
-     scfg-append!			;ignore
+     (declare (ignore scfg-append!))
      (lambda (address offset granularity)
        (receiver
 	(case granularity
@@ -551,13 +548,14 @@ USA.
 	  (expression-simplify type scfg-append!
 	    (lambda (type)
 	      (if use-pre/post-increment?
-		  (assign-to-temporary  
-		   (rtl:make-offset-address free
-					    (rtl:make-machine-constant (- nelements)))
+		  (assign-to-temporary
+		   (rtl:make-offset-address
+		    free
+		    (rtl:make-machine-constant (- nelements)))
 		   scfg-append!
 		   (lambda (temporary)
 		     (receiver (rtl:make-cons-pointer type temporary))))
-		  (assign-to-temporary 
+		  (assign-to-temporary
 		   (rtl:make-cons-pointer type free)
 		   scfg-append!
 		   (lambda (temporary)
@@ -578,7 +576,7 @@ USA.
 		    (scfg-append! (store-element! (car elements) offset)
 				  (loop (cdr elements)
 					(1+ offset))))))))
-	  
+
 	(let process ((elements elements)
 		      (offset 0)
 		      (chunk 1))
@@ -586,9 +584,9 @@ USA.
 	      (do-chunk elements
 			offset
 			(finish))
-	      (do-chunk (list-head elements chunk-size)
+	      (do-chunk (take elements chunk-size)
 			offset
-			(process (list-tail elements chunk-size)
+			(process (drop elements chunk-size)
 				 (+ offset chunk-size)
 				 (1+ chunk)))))))))
 
@@ -604,11 +602,11 @@ USA.
 	(begin
 	  (set! reg-list available-machine-registers)
 	  (set! value
-		(length (list-transform-positive reg-list
-			  (lambda (reg)
-			    (value-class/ancestor-or-self?
-			     (machine-register-value-class reg)
-			     value-class=word)))))
+		(length (filter (lambda (reg)
+				  (value-class/ancestor-or-self?
+				   (machine-register-value-class reg)
+				   value-class=word))
+				reg-list)))
 	  value)))))
 
 (define-expression-method 'TYPED-CONS:PROCEDURE
@@ -709,7 +707,7 @@ USA.
 				     operand1
 				     operand2
 				     overflow?))))))))
-
+
 (define-expression-method 'FIXNUM-1-ARG
   (lambda (receiver scfg-append! operator operand overflow?)
     (expression-simplify operand scfg-append!
@@ -737,5 +735,20 @@ USA.
 		       s-operand2
 		       overflow?))))))))
 
+(define-expression-method 'FLONUM-3-ARGS
+  (lambda (receiver scfg-append! operator operand1 operand2 operand3 overflow?)
+    (expression-simplify operand1 scfg-append!
+      (lambda (s-operand1)
+	(expression-simplify operand2 scfg-append!
+	  (lambda (s-operand2)
+	    (expression-simplify operand3 scfg-append!
+	      (lambda (s-operand3)
+		(receiver (rtl:make-flonum-3-args
+			   operator
+			   s-operand1
+			   s-operand2
+			   s-operand3
+			   overflow?))))))))))
+
 ;;; end EXPRESSION-SIMPLIFY package
 )
\ No newline at end of file
