@@ -3,7 +3,8 @@
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
     2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
+    2017, 2018, 2019, 2020, 2021, 2022 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -55,7 +56,7 @@ USA.
     (event-distributor/invoke! (ref-variable xml-mode-hook buffer) buffer)))
 
 (define xml-paragraph-separator
-  (let ((lwsp (rexp* (char-set #\space #\tab #\U+A0))))
+  (let ((lwsp (rexp* (char-set #\space #\tab #\xA0))))
     (rexp->regexp
      (rexp-sequence
       (rexp-optional lwsp
