@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019, 2020, 2021, 2022 Massachusetts Institute of
     Technology
 
 This file is part of MIT/GNU Scheme.
@@ -73,14 +74,14 @@ USA.
 		   (named)
 		   (constructor %make-group (buffer)))
   ;; The microcode file "edwin.h" depends on this structure being a
-  ;; named vector, and knows the indexes of the fields TEXT-DESCRIPTOR,
+  ;; named vector, and knows the indexes of the fields TEXT,
   ;; GAP-START, GAP-LENGTH, GAP-END, START-MARK, END-MARK, and
   ;; MODIFIED?.
-  text-descriptor
+  text
   (gap-start 0)
   (gap-length 0)
   (gap-end 0)
-  (marks '())
+  (marks (weak-list-set eq?) read-only #t)
   start-mark
   end-mark
   (writeable? #t)
@@ -96,8 +97,7 @@ USA.
   buffer
   (shrink-length 0)
   (text-properties #f)
-  (%hash-number #f)
-  %text)
+  (%hash-number #f))
 
 (define-integrable group-point group-%point)
 
@@ -113,17 +113,8 @@ USA.
     (set-group-%point! group (make-permanent-mark group 0 #t))
     group))
 
-(define (set-group-text! group text)
-  (without-interrupts
-   (lambda ()
-     (set-group-%text! group text)
-     (set-group-text-descriptor! group (external-string-descriptor text)))))
-
-(define-integrable (group-text group)
-  (group-%text group))
-
 (define (group-length group)
-  (fix:- (xstring-length (group-text group)) (group-gap-length group)))
+  (fix:- (string-length (group-text group)) (group-gap-length group)))
 
 (define-integrable (group-start-index group)
   (mark-index (group-start-mark group)))
@@ -202,7 +193,7 @@ USA.
 
 (define (group-hash-number group)
   (or (group-%hash-number group)
-      (let ((n (object-hash group)))
+      (let ((n (hash-object group)))
 	(set-group-%hash-number! group n)
 	n)))
 
@@ -306,17 +297,14 @@ USA.
 (define-structure (mark
 		   (constructor make-temporary-mark)
 		   (print-procedure
-		    (unparser/standard-method 'MARK
-		      (lambda (state mark)
-			(unparse-object state
-					(or (mark-buffer mark)
-					    (mark-group mark)))
-			(unparse-string state " ")
-			(unparse-object state (mark-index mark))
-			(unparse-string state
-					(if (mark-left-inserting? mark)
-					    " left"
-					    " right"))))))
+		    (standard-print-method 'MARK
+		      (lambda (mark)
+			(list (or (mark-buffer mark)
+				  (mark-group mark))
+			      (mark-index mark)
+			      (if (mark-left-inserting? mark)
+				  'left
+				  'right))))))
   ;; The microcode file "edwin.h" depends on the definition of this
   ;; structure.
   (group #f read-only #t)
@@ -359,19 +347,11 @@ USA.
 
 (define (make-permanent-mark group index left-inserting?)
   (let ((mark (make-temporary-mark group index left-inserting?)))
-    (set-group-marks! group
-		      (system-pair-cons (ucode-type weak-cons)
-					mark
-					(group-marks group)))
+    (weak-list-set-add! mark (group-marks group))
     mark))
 
 (define (mark-permanent! mark)
-  (let ((group (mark-group mark)))
-    (if (not (weak-memq mark (group-marks group)))
-	(set-group-marks! group
-			  (system-pair-cons (ucode-type weak-cons)
-					    mark
-					    (group-marks group)))))
+  (weak-list-set-add! mark (group-marks (mark-group mark)))
   mark)
 
 (define-integrable (mark-local-ref mark variable)
@@ -434,200 +414,25 @@ USA.
 (define-integrable (mark-absolute-end mark)
   (group-absolute-end (mark-group mark)))
 
-;;; The next few procedures are simple algorithms that are haired up
-;;; the wazoo for maximum speed.
-
 (define (clean-group-marks! group)
+  (weak-list-set-clean! (group-marks group)))
 
-  (define (scan-head marks)
-    (cond ((null? marks)
-	   (set-group-marks! group '()))
-	  ((not (system-pair-car marks))
-	   (scan-head (system-pair-cdr marks)))
-	  (else
-	   (set-group-marks! group marks)
-	   (scan-tail marks (system-pair-cdr marks)))))
-
-  (define (scan-tail previous marks)
-    (cond ((null? marks)
-	   unspecific)
-	  ((not (system-pair-car marks))
-	   (skip-nulls previous (system-pair-cdr marks)))
-	  (else
-	   (scan-tail marks (system-pair-cdr marks)))))
-
-  (define (skip-nulls previous marks)
-    (cond ((null? marks)
-	   (system-pair-set-cdr! previous '()))
-	  ((not (system-pair-car marks))
-	   (skip-nulls previous (system-pair-cdr marks)))
-	  (else
-	   (system-pair-set-cdr! previous marks)
-	   (scan-tail marks (system-pair-cdr marks)))))
-
-  (let ((marks (group-marks group)))
-    (cond ((null? marks)
-	   unspecific)
-	  ((not (system-pair-car marks))
-	   (scan-head (system-pair-cdr marks)))
-	  (else
-	   (scan-tail marks (system-pair-cdr marks))))))
-
 (define (mark-temporary! mark)
   ;; I'd think twice about using this one.
-  (let ((group (mark-group mark)))
+  (weak-list-set-delete! mark (group-marks (mark-group mark))))
 
-    (define (scan-head marks)
-      (if (null? marks)
-	  (set-group-marks! group '())
-	  (let ((mark* (system-pair-car marks)))
-	    (cond ((not mark*)
-		   (scan-head (system-pair-cdr marks)))
-		  ((eq? mark mark*)
-		   (set-group-marks! group (system-pair-cdr marks)))
-		  (else
-		   (set-group-marks! group marks)
-		   (scan-tail marks (system-pair-cdr marks)))))))
-
-    (define (scan-tail previous marks)
-      (if (not (null? marks))
-	  (let ((mark* (system-pair-car marks)))
-	    (cond ((not mark*)
-		   (skip-nulls previous (system-pair-cdr marks)))
-		  ((eq? mark mark*)
-		   (system-pair-set-cdr! previous marks))
-		  (else
-		   (scan-tail marks (system-pair-cdr marks)))))))
-
-    (define (skip-nulls previous marks)
-      (if (null? marks)
-	  (system-pair-set-cdr! previous '())
-	  (let ((mark* (system-pair-car marks)))
-	    (cond ((not mark*)
-		   (skip-nulls previous (system-pair-cdr marks)))
-		  ((eq? mark mark*)
-		   (system-pair-set-cdr! previous (system-pair-cdr marks)))
-		  (else
-		   (system-pair-set-cdr! previous marks)
-		   (scan-tail marks (system-pair-cdr marks)))))))
-
-    (let ((marks (group-marks group)))
-      (if (not (null? marks))
-	  (let ((mark* (system-pair-car marks)))
-	    (cond ((not mark*)
-		   (scan-head (system-pair-cdr marks)))
-		  ((eq? mark mark*)
-		   (set-group-marks! group (system-pair-cdr marks)))
-		  (else
-		   (scan-tail marks (system-pair-cdr marks)))))))))
-
 (define (find-permanent-mark group index left-inserting?)
+  (weak-list-set-find (if left-inserting?
+			  (lambda (mark)
+			    (and (mark-left-inserting? mark)
+				 (fix:= index (mark-index mark))))
+			  (lambda (mark)
+			    (and (not (mark-left-inserting? mark))
+				 (fix:= index (mark-index mark)))))
+		      (group-marks group)))
 
-  (define (scan-head marks)
-    (if (null? marks)
-	(begin
-	  (set-group-marks! group '())
-	  #f)
-	(let ((mark (system-pair-car marks)))
-	  (cond ((not mark)
-		 (scan-head (system-pair-cdr marks)))
-		((and (if (mark-left-inserting? mark)
-			  left-inserting?
-			  (not left-inserting?))
-		      (fix:= (mark-index mark) index))
-		 mark)
-		(else
-		 (set-group-marks! group marks)
-		 (scan-tail marks (system-pair-cdr marks)))))))
-
-  (define (scan-tail previous marks)
-    (and (not (null? marks))
-	 (let ((mark (system-pair-car marks)))
-	   (cond ((not mark)
-		  (skip-nulls previous (system-pair-cdr marks)))
-		 ((and (if (mark-left-inserting? mark)
-			   left-inserting?
-			   (not left-inserting?))
-		       (fix:= (mark-index mark) index))
-		  mark)
-		 (else
-		  (scan-tail marks (system-pair-cdr marks)))))))
-
-  (define (skip-nulls previous marks)
-    (if (null? marks)
-	(begin
-	  (system-pair-set-cdr! previous '())
-	  #f)
-	(let ((mark (system-pair-car marks)))
-	  (if (not mark)
-	      (skip-nulls previous (system-pair-cdr marks))
-	      (begin
-		(system-pair-set-cdr! previous marks)
-		(if (and (if (mark-left-inserting? mark)
-			     left-inserting?
-			     (not left-inserting?))
-			 (fix:= (mark-index mark) index))
-		    mark
-		    (scan-tail marks (system-pair-cdr marks))))))))
-
-  (let ((marks (group-marks group)))
-    (and (not (null? marks))
-	 (let ((mark (system-pair-car marks)))
-	   (cond ((not mark)
-		  (scan-head (system-pair-cdr marks)))
-		 ((and (if (mark-left-inserting? mark)
-			   left-inserting?
-			   (not left-inserting?))
-		       (fix:= (mark-index mark) index))
-		  mark)
-		 (else
-		  (scan-tail marks (system-pair-cdr marks))))))))
-
 (define (for-each-mark group procedure)
-
-  (define (scan-head marks)
-    (if (null? marks)
-	(set-group-marks! group '())
-	(let ((mark (system-pair-car marks))
-	      (rest (system-pair-cdr marks)))
-	  (if mark
-	      (begin
-		(set-group-marks! group marks)
-		(procedure mark)
-		(scan-tail marks rest))
-	      (scan-head rest)))))
-
-  (define (scan-tail previous marks)
-    (if (not (null? marks))
-	(let ((mark (system-pair-car marks))
-	      (rest (system-pair-cdr marks)))
-	  (if mark
-	      (begin
-		(procedure mark)
-		(scan-tail marks rest))
-	      (skip-nulls previous rest)))))
-
-  (define (skip-nulls previous marks)
-    (if (null? marks)
-	(system-pair-set-cdr! previous '())
-	(let ((mark (system-pair-car marks))
-	      (rest (system-pair-cdr marks)))
-	  (if mark
-	      (begin
-		(system-pair-set-cdr! previous marks)
-		(procedure mark)
-		(scan-tail marks rest))
-	      (skip-nulls previous rest)))))
-
-  (let ((marks (group-marks group)))
-    (if (not (null? marks))
-	(let ((mark (system-pair-car marks))
-	      (rest (system-pair-cdr marks)))
-	  (if mark
-	      (begin
-		(procedure mark)
-		(scan-tail marks rest))
-	      (scan-head rest))))))
+  (weak-list-set-for-each procedure (group-marks group)))
 
 ;;;; Regions
 
