@@ -3,7 +3,8 @@
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
     2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
+    2017, 2018, 2019, 2020, 2021, 2022 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -370,17 +371,14 @@ USA.
   (vector-set! inferior 4 redisplay-flags))
 
 (define-print-method %inferior?
-  (bracketed-print-method 'inferior
-    (lambda (inferior port)
-      (write-string " " port)
-      (write (inferior-window inferior) port)
-      (write-string " x,y=(" port)
-      (write (inferior-x-start inferior) port)
-      (write-string "," port)
-      (write (inferior-y-start inferior) port)
-      (write-string ")" port)
-      (if (inferior-needs-redisplay? inferior)
-	  (write-string " needs-redisplay" port)))))
+  (standard-print-method 'inferior
+    (lambda (inferior)
+      (cons* (inferior-window inferior)
+	     (list (inferior-x-start inferior)
+		   (inferior-y-start inferior))
+	     (if (inferior-needs-redisplay? inferior)
+		 (list 'needs-redisplay)
+		 '())))))
 
 (define (inferior-copy inferior)
   (%make-inferior (inferior-window inferior)
