@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019, 2020, 2021, 2022 Massachusetts Institute of
     Technology
 
 This file is part of MIT/GNU Scheme.
@@ -31,6 +32,8 @@ USA.
 
 #include "scheme.h"
 #include "prims.h"
+#include "bits.h"
+#include "fixnum.h"
 
 static long
 arg_fixnum (int n)
@@ -208,16 +211,9 @@ DEFINE_PRIMITIVE ("FIXNUM-QUOTIENT", Prim_fixnum_quotient, 2, 2, 0)
   {
     long numerator = (arg_fixnum (1));
     long denominator = (arg_fixnum (2));
-    FIXNUM_RESULT
-      ((denominator > 0)
-       ? ((numerator < 0)
-	  ? (- ((- numerator) / denominator))
-	  : (numerator / denominator))
-       : (denominator < 0)
-       ? ((numerator < 0)
-	  ? ((- numerator) / (- denominator))
-	  : (- (numerator / (- denominator))))
-       : (error_bad_range_arg (2), 0));
+    if (denominator == 0)
+      error_bad_range_arg (2);
+    FIXNUM_RESULT (FIXNUM_QUOTIENT (numerator, denominator));
   }
 }
 
@@ -227,16 +223,9 @@ DEFINE_PRIMITIVE ("FIXNUM-REMAINDER", Prim_fixnum_remainder, 2, 2, 0)
   {
     long numerator = (arg_fixnum (1));
     long denominator = (arg_fixnum (2));
-    FIXNUM_RESULT
-      ((denominator > 0)
-       ? ((numerator < 0)
-	  ? (- ((- numerator) % denominator))
-	  : (numerator % denominator))
-       : (denominator < 0)
-       ? ((numerator < 0)
-	  ? (- ((- numerator) % (- denominator)))
-	  : (numerator % (- denominator)))
-       : (error_bad_range_arg (2), 0));
+    if (denominator == 0)
+      error_bad_range_arg (2);
+    FIXNUM_RESULT (FIXNUM_REMAINDER (numerator, denominator));
   }
 }
 
@@ -292,18 +281,33 @@ DEFINE_PRIMITIVE ("FIXNUM-LSH", Prim_fixnum_lsh, 2, 2, 0)
 {
   PRIMITIVE_HEADER (2);
   {
-    unsigned long x = (arg_unsigned_fixnum (1));
+    long x = (arg_fixnum (1));
     long y = (arg_fixnum (2));
-    unsigned long z;
 
-    if (y < 0)
-      z = (((-y) > ((long) DATUM_LENGTH)) ? 0 : (x >> (-y)));
-    else
-      z = ((y > ((long) DATUM_LENGTH)) ? 0 : (x << y));
-    LOGICAL_RESULT (z);
+    LOGICAL_RESULT (FIXNUM_LSH (x, y));
   }
 }
 
+DEFINE_PRIMITIVE ("FXBIT-COUNT", Prim_fxbit_count, 1, 1, 0)
+{
+  PRIMITIVE_HEADER (1);
+  long n = (arg_fixnum(1));
+  FIXNUM_RESULT (ulong_bit_count ((unsigned long) ((n < 0) ? (~n) : n)));
+}
+
+DEFINE_PRIMITIVE ("FXLENGTH", Prim_fxlength, 1, 1, 0)
+{
+  PRIMITIVE_HEADER (1);
+  long n = (arg_fixnum(1));
+  FIXNUM_RESULT (ulong_length_in_bits ((unsigned long) ((n < 0) ? ~n : n)));
+}
+
+DEFINE_PRIMITIVE ("FXFIRST-SET-BIT", fxfirst_set_bit, 1, 1, 0)
+{
+  PRIMITIVE_HEADER (1);
+  FIXNUM_RESULT (ulong_first_set_bit ((unsigned long) (arg_fixnum(1))));
+}
+
 DEFINE_PRIMITIVE ("FIXNUM->FLONUM", Prim_fixnum_to_flonum, 1, 1,
 		  "(FIXNUM)\n\
 Equivalent to (INTEGER->FLONUM FIXNUM 2)")
