@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019, 2020, 2021, 2022 Massachusetts Institute of
     Technology
 
 This file is part of MIT/GNU Scheme.
@@ -38,8 +39,7 @@ arg_terminal (int argument_number)
   Tchannel channel = (arg_channel (argument_number));
   enum channel_type type = (OS_channel_type (channel));
   if (! ((type == channel_type_terminal)
-	 || (type == channel_type_unix_pty_master)
-	 || (type == channel_type_os2_console)))
+	 || (type == channel_type_unix_pty_master)))
     error_bad_range_arg (argument_number);
   return (channel);
 }
