@@ -2,7 +2,9 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019, 2020, 2021, 2022 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -42,16 +44,16 @@ USA.
 		      (environment-lookup system-global-environment name)))
 		 (if (not (memq (microcode-type/code->name
 				 (object-type object))
-				'(BIGNUM
-				  CHARACTER
-				  CONSTANT
-				  FALSE
-				  FIXNUM
-				  FLONUM
-				  INTERNED-SYMBOL
-				  RATNUM
-				  RECNUM
-				  UNINTERNED-SYMBOL)))
+				'(bignum
+				  character
+				  constant
+				  false
+				  fixnum
+				  flonum
+				  interned-symbol
+				  ratnum
+				  recnum
+				  uninterned-symbol)))
 		     (error "USUAL-INTEGRATIONS: not a constant" name))
 		 (constant->integration-info object)))
 	     usual-integrations/constant-names))
