@@ -3,7 +3,8 @@
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
     2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
+    2017, 2018, 2019, 2020, 2021, 2022 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -466,4 +467,29 @@ USA.
 
 (define-test 'SHIFT-RIGHT/TOO-MANY
   (lambda ()
-    (assert-= (shift-right (identity-procedure 1234567) 100) 0)))
\ No newline at end of file
+    (assert-= (shift-right (identity-procedure 1234567) 100) 0)))
+
+(define-test 'bitwise-and/none (lambda () (assert-= (bitwise-and) -1)))
+(define-test 'bitwise-and/one (lambda () (assert-= (bitwise-and 123) 123)))
+(define-test 'bitwise-xor/none (lambda () (assert-= (bitwise-xor) 0)))
+(define-test 'bitwise-xor/one (lambda () (assert-= (bitwise-xor 123) 123)))
+(define-test 'bitwise-ior/none (lambda () (assert-= (bitwise-ior) 0)))
+(define-test 'bitwise-ior/one (lambda () (assert-= (bitwise-ior 123) 123)))
+(define-test 'bitwise-eqv/none (lambda () (assert-= (bitwise-eqv) -1)))
+(define-test 'bitwise-eqv/one (lambda () (assert-= (bitwise-eqv 123) 123)))
+
+(define-test 'bitwise-and/three
+  (lambda ()
+    (assert-= (bitwise-and #b1110 #b1101 #b1011) #b1000)))
+
+(define-test 'bitwise-xor/three
+  (lambda ()
+    (assert-= (bitwise-xor #b1000 #b1100 #b1110) #b1010)))
+
+(define-test 'bitwise-ior/three
+  (lambda ()
+    (assert-= (bitwise-ior #b1000 #b0100 #b0010) #b1110)))
+
+(define-test 'bitwise-eqv/three
+  (lambda ()
+    (assert-= (bitwise-eqv #b1000 #b1100 #b1110) #b1010)))
\ No newline at end of file
